// -*- C++ -*-
/*!
 * @file  TileGenerator.h
 * @brief 画像付加データDBから画像付加データを取得しタイル画像を作成する
 * @date  $Date$
 *
 * @author 株式会社 東日本計算センター
 *
 * New BSD
 *
 * $Id$
 */

#ifndef TILEGENERATOR_H
#define TILEGENERATOR_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">
#include "TileGeneratorSVC_impl.h"
#include "local_basic_type.h"
#include "VersatileOverheadSystem.h"

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">
#include "AdditinalDataDBControlStub.h"
#include "MapDataDBControlStub.h"
#include "json.h"
#include "writer.h"

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="port_stub_h">
// </rtc-template>

using namespace RTC;

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>

/*====================================================
	struct定義
===================================================*/

//算出座標データ構造体
struct CalcCoord_st{
	double_type dStartLatitude;				//起点緯度
	double_type dStartLongitude;			//起点経度
	double_type dEndLatitude;				//終点緯度
	double_type dEndLongitude;				//終点経度
};

/*====================================================
	マクロ定義
===================================================*/
//マトリクスモード
enum MATRIX_MODE_NUM{
	MODE_IDLE = 0,						//アイドル状態
	MODE_ADD_DATA_GET,					//画像付加データ取得中
	MODE_SPLIT_DISP,					//タイル分割確認画面表示中
	MODE_MAPDB_REGIST,					//地図データDB登録中
	MODE_MAX,
};
#define MODE_NO_CHG		0xFFFF			//モード切替なし

//マトリクストリガ
enum MATRIX_TRIGGER_NUM{
	TRG_ADDDB_GET_ADD_DATA_SUCCESS = 0,	//地図画像付加情報取得正常終了
	TRG_ADDDB_GET_ADD_DATA_ERROR,		//地図画像付加情報取得異常終了
	TRG_MAPDB_REGIST_SUCCESS,			//地図画像付加データ取得正常終了
	TRG_MAPDB_REGIST_ERROR,				//地図画像付加データ取得異常終了
	TRG_REQ_TILE_GENERATE,				//タイル生成要求
	TRG_REQ_SPLIT_DISP_ON,				//タイル分割確認画面表示要求
	TRG_REQ_SPLIT_DISP_OFF,				//タイル分割確認画面消去要求
	TRG_REQ_SPLIT_DISP_RENEW,			//タイル分割確認画面更新要求
	TRG_MAX,
};

#define TRG_NON			0xFFFF			//トリガなし

//マトリクスアクション
enum MATRIX_ACT_NUM{
	ACT_TILE_GENERATE_START = 0,		//地図作成開始
	ACT_TILE_SPLIT,						//タイル分割処理（確認画面表示なども行う）
	ACT_TILE_GENERATE_COMP,				//画像付加データ取得
	ACT_TILE_GENERATE_ERROR,			//タイル分割失敗
	ACT_SPLIT_DISP_RENEW,				//タイル分割確認画面更新
	ACT_SPLIT_DISP_OFF,					//タイル分割確認画面消去
	ACT_MAX,
};
#define ACT_NON			0xFFFF			//アクションなし

//タイルサイズ
#define TILE_SIZE		256				//タイル幅/高さのサイズ

#define UWORD_TYPE_MAX	0xFFFFFFFF

#define ALT_CONV_TBL_MAX	4			

/*!
 * @class TileGenerator
 * @brief 画像付加データDBから画像付加データを取得しタイル画像を作成する
 *
 * 別紙"機能仕様書_タイル生成RTC"参照
 *
 * 別紙"機能仕様書_タイル生成RTC"参照
 *
 * 別紙"機能仕様書_タイル生成RTC"参照
 *
 * 別紙"機能仕様書_タイル生成RTC"参照
 *
 */
class TileGenerator
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  TileGenerator(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~TileGenerator();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   * タイル生成RTCの初期化処理
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   * タイル生成RTCの終了処理
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   * タイル生成RTCの開始処理
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   * タイル生成RTCの停止処理
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   * タイル生成RTCの周期処理
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   * タイル生成RTCのエラー遷移処理
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre -
   * @post -
   * 
   */
   virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /*!
   * 入力画像のサイズを拡大、縮小する
   * - Name: Image_Resize fImage_Resize
   * - DefaultValue: 1
   * - Unit: %
   * - Range: 0<x<=2.0
   * - Constraint: -
   */
  float m_fImage_Resize;
  /*!
   * タイル分割時の開始位置のオフセット(幅)
   * - Name: Offset_Width lOffset_Width
   * - DefaultValue: 0
   * - Unit: pixel
   * - Range: 0<=x<=256
   * - Constraint: -
   */
  long int m_lOffset_Width;
  /*!
   * タイル分割時の開始位置のオフセット(高さ)
   * - Name: Offset_Height lOffset_Height
   * - DefaultValue: 0
   * - Unit: pixel
   * - Range: 0<=x<=256
   * - Constraint: -
   */
  long int m_lOffset_Height;
  /*!
   * タイルX軸の起点座標
   * - Name: TileX_Coord_Start lTileX_Coord_Start
   * - DefaultValue: 0
   * - Range: 0<=x<=4294967294
   * - Constraint: -
   */
  long int m_lTileX_Coord_Start;
  /*!
   * タイルY軸の起点座標
   * - Name: TileY_Coord_Start lTileY_Coord_Start
   * - DefaultValue: 0
   * - Range: 0<=x<=4294967294
   * - Constraint: -
   */
  long int m_lTileY_Coord_Start;
  /*!
   * ズームレベル18に対応した高度下限
   * - Name: ZLvl18_Alt_Min lZLvl18_Alt_Min
   * - DefaultValue: 30
   * - Unit: m
   * - Range: 0<=x<=150
   * - Constraint: -
   */
  long int m_lZLvl18_Alt_Min;
  /*!
   * ズームレベル18に対応した高度上限
   * - Name: ZLvl18_Alt_Max lZLvl18_Alt_Max
   * - DefaultValue: 49
   * - Unit: m
   * - Range: 0<=x<=150
   * - Constraint: -
   */
  long int m_lZLvl18_Alt_Max;
  /*!
   * ズームレベル19に対応した高度下限
   * - Name: ZLvl19_Alt_Min lZLvl19_Alt_Min
   * - DefaultValue: 50
   * - Unit: m
   * - Range: 0<=x<=150
   * - Constraint: -
   */
  long int m_lZLvl19_Alt_Min;
  /*!
   * ズームレベル19に対応した高度上限
   * - Name: ZLvl19_Alt_Max lZLvl19_Alt_Max
   * - DefaultValue: 69
   * - Unit: m
   * - Range: 0<=x<=150
   * - Constraint: -
   */
  long int m_lZLvl19_Alt_Max;
  /*!
   * ズームレベル20に対応した高度下限
   * - Name: ZLvl20_Alt_Min lZLvl20_Alt_Min
   * - DefaultValue: 70
   * - Unit: m
   * - Range: 0<=x<=150
   * - Constraint: -
   */
  long int m_lZLvl20_Alt_Min;
  /*!
   * ズームレベル20に対応した高度上限
   * - Name: ZLvl20_Alt_Max lZLvl20_Alt_Max
   * - DefaultValue: 89
   * - Unit: m
   * - Range: 0<=x<=150
   * - Constraint: -
   */
  long int m_lZLvl20_Alt_Max;
  /*!
   * ズームレベル21に対応した高度下限
   * - Name: ZLvl21_Alt_Min lZLvl21_Alt_Min
   * - DefaultValue: 90
   * - Unit: m
   * - Range: 0<=x<=150
   * - Constraint: -
   */
  long int m_lZLvl21_Alt_Min;
  /*!
   * ズームレベル21に対応した高度上限
   * - Name: ZLvl21_Alt_Max lZLvl21_Alt_Max
   * - DefaultValue: 110
   * - Unit: m
   * - Range: 0<=x<=150
   * - Constraint: -
   */
  long int m_lZLvl21_Alt_Max;
  /*!
   * 入力画像撮影時の対角角度
   * - Name: View_Angle dView_Angle
   * - DefaultValue: 31.5
   * - Unit: 角度
   * - Range: 0<x<=180.0
   * - Constraint: -
   */
  double m_dView_Angle;

  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  RTC::TimedStringSeq m_inCoordAdditinalData;
  /*!
   * 座標指定画像付加データをjyson形式で出力
   * - Type: RTC::TimedStringSeq
   * - Number: 100
   * - Semantics: -
   * - Unit: -
   * - Frequency: タイル生成時(座標指定画像付加データの取得要求を行
   *              った時)
   * - Operation Cycle: タイル生成時(座標指定画像付加データの取得要
   *                    求を行った時)
   */
  InPort<RTC::TimedStringSeq> m_inCoordAdditinalDataIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  /*!
   * タイル生成RTCへのリクエスト関連のインターフェース
   * Interface: タイル生成要求などのインターフェースを提供する
   */
  RTC::CorbaPort m_TileGenerateReqServPort;
  /*!
   * タイル生成RTCのステータス関連のインターフェース
   * Interface: 動作完了通知などのインターフェースを提供する
   */
  RTC::CorbaPort m_TileGenerateStsServPort;
  /*!
   * 画像付加データDB制御RTCへのリクエスト関連のインターフェース
   * Interface: 座標指定画像付加データ取得要求などのインターフェー
   *            スを提供する
   */
  RTC::CorbaPort m_AdditionalDBReqServPort;
  /*!
   * 画像付加データDB制御RTCのステータス関連のインターフェース
   * Interface: 動作完了通知などのインターフェースを提供する
   */
  RTC::CorbaPort m_AdditionalDBStsServPort;
  /*!
   * 地図データDB制御RTCへのリクエスト関連のインターフェース
   * Interface: 地図データ登録要求などのインターフェースを提供する
   */
  RTC::CorbaPort m_MapDataDBReqServPort;
  /*!
   * 地図データDB制御RTCのステータス関連のインターフェース
   * Interface: 動作完了通知などのインターフェースを提供する
   */
  RTC::CorbaPort m_MapDataDBStsServPort;
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  /*!
   * タイル画像を生成するための要求を行う
   * - Argument:      別紙"機能仕様書_タイル生成RTC"参照
   * - Return Value:  別紙"機能仕様書_タイル生成RTC"参照
   * - Exception:     なし
   * - PreCondition:  -
   * - PostCondition: -
   */
  TileGenerateReqMakeTileSVC_impl m_TileGenerateReqMakeTileProv;
  /*!
   * タイル生成RTCの動作完了通知を行う
   * - Argument:      別紙"機能仕様書_タイル生成RTC"参照
   * - Return Value:  別紙"機能仕様書_タイル生成RTC"参照
   * - Exception:     なし
   * - PreCondition:  -
   * - PostCondition: -
   */
  TileGenerateNoticeSVC_impl m_TileGenerateNoticeProv;
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  /*!
   * 座標指定画像付加データの取得要求を行う
   * - Argument:      別紙"機能仕様書_タイル生成RTC"参照
   * - Return Value:  別紙"機能仕様書_タイル生成RTC"参照
   * - Exception:     なし
   * - PreCondition:  -
   * - PostCondition: -
   */
  RTC::CorbaConsumer<AdditionalDBReqServ::CoordinateDataGet> m_CoordinateDataGetCons;
  /*!
   * 画像付加データDB制御RTCの動作完了通知を行う
   * - Argument:      別紙"機能仕様書_タイル生成RTC"参照
   * - Return Value:  別紙"機能仕様書_タイル生成RTC"参照
   * - Exception:     なし
   * - PreCondition:  -
   * - PostCondition: -
   */
  RTC::CorbaConsumer<AdditionalDBStsServ::AddDBNotice> m_AddDBNoticeCons;
  /*!
   * 地図データDB登録要求を行う
   * - Argument:      別紙"機能仕様書_タイル生成RTC"参照
   * - Return Value:  別紙"機能仕様書_タイル生成RTC"参照
   * - Exception:     なし
   * - PreCondition:  -
   * - PostCondition: -
   */
  RTC::CorbaConsumer<MapDataDBReqServ::MapDBReqRegistAddData> m_MapDBReqRegistAddDataCons;
  /*!
   * 地図データDB制御RTCの動作完了通知を行う
   * - Argument:      別紙"機能仕様書_タイル生成RTC"参照
   * - Return Value:  別紙"機能仕様書_タイル生成RTC"参照
   * - Exception:     なし
   * - PreCondition:  -
   * - PostCondition: -
   */
  RTC::CorbaConsumer<MapDataDBStsServ::MapDataDBNotice> m_MapDataDBNoticeCons;
  
  // </rtc-template>

 private:
	 bool_type blTileGeneMatrixTrigerSet(void);
	 uhword_type uhwTileGeneMatrixCheck(void);
	 uhword_type uhwTileGeneProcExe(void);
	 bool_type blAddDBNoticeTrigCheck(uhword_type &);
	 bool_type blMapDBNoticeTrigCheck(uhword_type &);
	 bool_type blTileGeneTrigCheck(uhword_type &);
	 uhword_type uhwProcTileGenerateStart(void);
	 uhword_type uhwProcTileSplit(void);
	 uhword_type uhwProcTileGenerateComp(void);
	 uhword_type uhwProcTileGenerateError(void);
	 uhword_type uhwProcSplitDispRenew(void);
	 uhword_type uhwProcSplitDispOff(void);
	 bool_type blAddDBReqCoordDataGetConsChk(void);
	 bool_type blAddDBNoticeConsChk(void);
	 bool_type blMapDBReqRegistConsChk(void);
	 bool_type blMapDBNoticeConsChk(void);
	 uhword_type uhwAddDataRamSet(std::string, uhword_type);
	 uhword_type uhwImgAddDataRangeCheck(ImgAddDBAddData_st);
	 std::string strUTF8toSjis(std::string);
	 std::string strSjistoUTF8(std::string);
	 uhword_type uhwImgResize(void);
	 uhword_type uhwSplitImgDisp(void);
	 uhword_type uhwSplitImgCreate(CalcCoord_st);
	 std::string strFolderCreate(void);
	 uhword_type uhwInputImageCoordCalc(CalcCoord_st &);
	 void CoordHextoDec(double_type, uword_type &, uword_type &, double_type &);
	 double_type dCoordDectoHex(uword_type, uword_type, double_type);
	 uhword_type uhwJsonFileCreate(void);
	 uhword_type uhwJsonDataMake(CalcCoord_st, std::string, std::string, uword_type, uword_type);
	 uhword_type uhwJsonFileAdd(void);
	 std::string strGetNowDate(void);
	 uhword_type uhwMapDBReqRegist(void);
	 void TileGeneratorRamInit(void);
	 std::string strImgJsonDataReplace(std::string);
	 void TileGeneModeSet(uhword_type);
	 void TileGenerateNoticeSet(uword_type, uword_type);
  
	 //マトリクス制御用
	 uhword_type m_uhwMatrixTriger;			//マトリクス内部トリガ
	 uhword_type m_uhwMatrixMode;			//マトリクスモード
	 uhword_type m_uhwMatrixActID;			//マトリクスアクションID

	 //完了通知保持
	 uword_type m_uwAddDBNotice;			//画像付加データDB完了通知
	 uword_type m_uwAddDBParam;				//画像付加データDB完了通知パラメータ
	 uword_type m_uwMapDBNotice;			//地図データDB完了通知
	 uword_type m_uwMapDBParam;				//地図データDB完了通知パラメータ
  
	 //タイル生成要求パラメータ
	 uword_type m_uwGenerateMode;			//タイル生成モード
	 uword_type m_uwRobotCategory;			//ロボットカテゴリ
	 uword_type m_uwGroupId;				//グループID
	 double_type m_dStartLatitude;			//起点緯度
	 double_type m_dStartLongitude;			//起点経度
	 double_type m_dEndLatitude;			//終点緯度
	 double_type m_dEndLongitude;			//終点経度
	 uword_type m_uwZoomlevel;				//ズームレベル

	 //画像付加データ
	 ImgAddDBAddData_st m_stImgAddData[PLAN_LIST_NUM_MAX];

	 //Jsonデータ作成
	 Json::Value m_JsonData;				//Jsonデータ格納
	 uhword_type m_uhwJsonMakeCnt;			//作成カウンタ
};


extern "C"
{
  DLL_EXPORT void TileGeneratorInit(RTC::Manager* manager);
};

#endif // TILEGENERATOR_H
