// -*-C++-*-
/*!
 * @file  TileGeneratorSVC_impl.cpp
 * @brief Service implementation code of TileGenerator.idl
 *
 * @author 株式会社 東日本計算センター
 *
 * New BSD
 *
 */

#include "TileGeneratorSVC_impl.h"
#include "VersatileOverheadSystem.h"
#include <iostream>

/*
 * Example implementational code for IDL interface TileGenerateReqServ::TileGenerateReqMakeTile
 */
TileGenerateReqMakeTileSVC_impl::TileGenerateReqMakeTileSVC_impl()
{
	m_ulGenerateMode = TILE_MODE_INIT;
	m_ulRobotCategory = ROBOT_CATEGORY_INIT;
	m_ulGroupId = GROUP_ID_NON;
	m_dStartLatitude = 0;
	m_dStartLongitude = 0;
	m_dEndLatitude = 0;
	m_dEndLongitude = 0;
	m_ulZoomlevel = ZOOM_LEVEL_DEF;
}


TileGenerateReqMakeTileSVC_impl::~TileGenerateReqMakeTileSVC_impl()
{
  // Please add extra destructor code here.
}


/*
 * Methods corresponding to IDL attributes and operations
 */
CORBA::UShort TileGenerateReqMakeTileSVC_impl::usSetTileGenerateReqMakeTile(CORBA::ULong ulGenerateMode, CORBA::ULong ulRobotCategory, CORBA::ULong ulGroupId, CORBA::Double dStartLatitude, CORBA::Double dStartLongitude, CORBA::Double dEndLatitude, CORBA::Double dEndLongitude, CORBA::ULong ulZoomlevel)
{
	if((ulGenerateMode > TILE_MODE_INIT) &&
		(ulGenerateMode < TILE_MODE_MAX) &&
		(ulRobotCategory > ROBOT_CATEGORY_INIT ) &&
		(ulRobotCategory < ROBOT_CATEGORY_MAX) &&
		(ulGroupId > GROUP_ID_NON) &&
		(dStartLatitude > 0) &&
		(dStartLongitude > 0) &&
		(dEndLatitude > 0) &&
		(dEndLongitude > 0) &&
		((ulZoomlevel >= ZOOM_LEVEL_MIN) && (ulZoomlevel <= ZOOM_LEVEL_MAX))){

		m_ulGenerateMode = ulGenerateMode;
		m_ulRobotCategory = ulRobotCategory;
		m_ulGroupId = ulGroupId;
		m_dStartLatitude = dStartLatitude;
		m_dStartLongitude = dStartLongitude;
		m_dEndLatitude = dEndLatitude;
		m_dEndLongitude = dEndLongitude;
		m_ulZoomlevel = ulZoomlevel;
	}
	else{
		std::cout << "[error]TileGenerateReqMakeTileSVC_impl::usSetTileGenerateReqMakeTile():ParamErr = " << ulGenerateMode
			<< ", " << ulRobotCategory
			<< ", " << ulGroupId
			<< ", " << dStartLatitude
			<< ", " << dStartLongitude
			<< ", " << dEndLatitude
			<< ", " << dEndLongitude
			<< ", " << ulZoomlevel
			<< std::endl;
		return RESULT_NACK;
	}
	
	return RESULT_ACK;
}

void TileGenerateReqMakeTileSVC_impl::GetTileGenerateReqMakeTile(CORBA::ULong& ulGenerateMode, CORBA::ULong& ulRobotCategory, CORBA::ULong& ulGroupId, CORBA::Double& dStartLatitude, CORBA::Double& dStartLongitude, CORBA::Double& dEndLatitude, CORBA::Double& dEndLongitude, CORBA::ULong& ulZoomlevel)
{
	if((&ulGenerateMode != NULL) && (&ulRobotCategory != NULL) && (&ulGroupId != NULL) && (&dStartLatitude != NULL) && (&dStartLongitude != NULL) &&
		(&dEndLatitude != NULL) && (&dEndLongitude != NULL) && (&ulZoomlevel != NULL)){
		ulGenerateMode = m_ulGenerateMode;
		ulRobotCategory = m_ulRobotCategory;
		ulGroupId = m_ulGroupId;
		dStartLatitude = m_dStartLatitude;
		dStartLongitude = m_dStartLongitude;
		dEndLatitude = m_dEndLatitude;
		dEndLongitude = m_dEndLongitude;
		ulZoomlevel = m_ulZoomlevel;
	}
	else{
		std::cout << "[error]TileGenerateReqMakeTileSVC_impl::GetTileGenerateReqMakeTile()" << std::endl;
	}
}

void TileGenerateReqMakeTileSVC_impl::InitTileGenerateReqMakeTile(void)
{
	m_ulGenerateMode = TILE_MODE_INIT;
	m_ulRobotCategory = ROBOT_CATEGORY_INIT;
	m_ulGroupId = GROUP_ID_NON;
	m_dStartLatitude = 0;
	m_dStartLongitude = 0;
	m_dEndLatitude = 0;
	m_dEndLongitude = 0;
	m_ulZoomlevel = ZOOM_LEVEL_DEF;
}


// End of example implementational code

/*
 * Example implementational code for IDL interface TileGenerateStsServ::TileGenerateNotice
 */
TileGenerateNoticeSVC_impl::TileGenerateNoticeSVC_impl()
{
	m_ulNotice = NOTICE_INIT;
	m_ulParam = NOTICE_PARAM_INIT;
}


TileGenerateNoticeSVC_impl::~TileGenerateNoticeSVC_impl()
{
  // Please add extra destructor code here.
}


/*
 * Methods corresponding to IDL attributes and operations
 */
CORBA::UShort TileGenerateNoticeSVC_impl::usSetTileGenerateNotice(CORBA::ULong ulNotice, CORBA::ULong ulParam)
{
	m_ulNotice = ulNotice;
	m_ulParam = ulParam;

	return RESULT_ACK;
}

void TileGenerateNoticeSVC_impl::GetTileGenerateNotice(CORBA::ULong& ulNotice, CORBA::ULong& ulParam)
{
	if((&ulNotice != NULL) && (&ulParam != NULL)){
		ulNotice = m_ulNotice;
		ulParam = m_ulParam;
	}
	else{
		std::cout << "[error]TileGenerateNoticeSVC_impl::GetTileGenerateNotice()" << std::endl;
	}
}

void TileGenerateNoticeSVC_impl::InitTileGenerateNotice(void)
{
	m_ulNotice = NOTICE_INIT;
	m_ulParam = NOTICE_PARAM_INIT;
}


// End of example implementational code



